@@ -1,46 +1,55 @@
 package Net::GitHub::V3::Query;
 
-use Any::Moose 'Role';
-
-our $VERSION = '0.40';
+our $VERSION = '0.67';
 our $AUTHORITY = 'cpan:FAYLAND';
 
 use URI;
 use JSON::Any;
-use WWW::Mechanize::GZip;
 use MIME::Base64;
+use LWP::UserAgent;
 use HTTP::Request;
 use Carp qw/croak/;
 use URI::Escape;
+use Types::Standard qw(Str Bool InstanceOf);
+
+use Moo::Role;
 
 # configurable args
 
 # Authentication
-has 'login'  => ( is => 'rw', isa => 'Str', predicate => 'has_login' );
-has 'pass'  => ( is => 'rw', isa => 'Str', predicate => 'has_pass' );
-has 'access_token' => ( is => 'rw', isa => 'Str', predicate => 'has_access_token' );
+has 'login'  => ( is => 'rw', isa => Str, predicate => 'has_login' );
+has 'pass'  => ( is => 'rw', isa => Str, predicate => 'has_pass' );
+has 'access_token' => ( is => 'rw', isa => Str, predicate => 'has_access_token' );
 
 # return raw unparsed JSON
-has 'raw_string' => (is => 'rw', isa => 'Bool', default => 0);
-has 'raw_response' => (is => 'rw', isa => 'Bool', default => 0);
+has 'raw_string' => (is => 'rw', isa => Bool, default => 0);
+has 'raw_response' => (is => 'rw', isa => Bool, default => 0);
 
 has 'api_url' => (is => 'ro', default => 'https://api.github.com');
-has 'api_throttle' => ( is => 'rw', isa => 'Bool', default => 1 );
+has 'api_throttle' => ( is => 'rw', isa => Bool, default => 1 );
+
+has 'upload_url' => (is => 'ro', default => 'https://uploads.github.com');
+
+# pagination
+has 'next_url'  => ( is => 'rw', isa => Str, predicate => 'has_next_page',  clearer => 'clear_next_url' );
+has 'last_url'  => ( is => 'rw', isa => Str, predicate => 'has_last_page',  clearer => 'clear_last_url' );
+has 'first_url' => ( is => 'rw', isa => Str, predicate => 'has_first_page', clearer => 'clear_first_url' );
+has 'prev_url'  => ( is => 'rw', isa => Str, predicate => 'has_prev_page',  clearer => 'clear_prev_url' );
 
 # Error handle
-has 'RaiseError' => ( is => 'rw', isa => 'Bool', default => 1 );
+has 'RaiseError' => ( is => 'rw', isa => Bool, default => 1 );
 
 # optional
-has 'u'  => (is => 'rw', isa => 'Str');
-has 'repo' => (is => 'rw', isa => 'Str');
+has 'u'  => (is => 'rw', isa => Str);
+has 'repo' => (is => 'rw', isa => Str);
 
-has 'is_main_module' => (is => 'ro', isa => 'Bool', default => 0);
+has 'is_main_module' => (is => 'ro', isa => Bool, default => 0);
 sub set_default_user_repo {
     my ($self, $user, $repo) = @_;
-    
+
     $self->u($user);
     $self->repo($repo);
-    
+
     # need apply to all sub modules
     if ($self->is_main_module) {
         if ($self->is_repos_init) {
@@ -63,7 +72,7 @@ sub set_default_user_repo {
 sub args_to_pass {
     my $self = shift;
     my $ret;
-    foreach my $col ('login', 'pass', 'access_token', 'raw_string', 'raw_response', 'api_url', 'api_throttle', 'u', 'repo') {
+    foreach my $col ('login', 'pass', 'access_token', 'raw_string', 'raw_response', 'api_url', 'api_throttle', 'u', 'repo', 'next_url', 'last_url', 'first_url', 'prev_url') {
         my $v = $self->$col;
         $ret->{$col} = $v if defined $v;
     }
@@ -71,16 +80,13 @@ sub args_to_pass {
 }
 
 has 'ua' => (
-    isa     => 'WWW::Mechanize::GZip',
+    isa     => InstanceOf['LWP::UserAgent'],
     is      => 'ro',
     lazy    => 1,
     default => sub {
-        my $self = shift;
-        return WWW::Mechanize::GZip->new(
+        LWP::UserAgent->new(
             agent       => "perl-net-github $VERSION",
             cookie_jar  => {},
-            stack_depth => 1,
-            autocheck   => 0,
             keep_alive  => 4,
             timeout     => 60,
         );
@@ -89,7 +95,7 @@ has 'ua' => (
 
 has 'json' => (
     is => 'ro',
-    isa => 'JSON::Any',
+    isa => InstanceOf['JSON::Any'],
     lazy => 1,
     default => sub {
         return JSON::Any->new( utf8 => 1 );
@@ -98,7 +104,7 @@ has 'json' => (
 
 sub query {
     my $self = shift;
-    
+
     # fix ARGV, not sure if it's the good idea
     my @args = @_;
     if (@args == 1) {
@@ -121,10 +127,13 @@ sub query {
     }
 
     $url = $self->api_url . $url unless $url =~ /^https\:/;
-    
+
+    print STDERR ">>> $request_method $url\n" if $ENV{NG_DEBUG};
     my $req = HTTP::Request->new( $request_method, $url );
+    $req->accept_decodable;
     if ($data) {
         my $json = $self->json->objToJson($data);
+        print STDERR ">>> $data\n" if $ENV{NG_DEBUG} and $ENV{NG_DEBUG} > 1;
         $req->content($json);
     }
     $req->header( 'Content-Length' => length $req->content );
@@ -139,11 +148,12 @@ sub query {
             < ($res->header('x-ratelimit-limit') || 60) / 2);
     }
 
-    return $ua->res if $self->raw_response;
-    return $ua->content if $self->raw_string;
+    print STDERR "<<< " . $res->decoded_content . "\n" if $ENV{NG_DEBUG} and $ENV{NG_DEBUG} > 1;
+    return $res if $self->raw_response;
+    return $res->decoded_content if $self->raw_string;
 
-    if ($res->header('Content-Type') and $res->header('Content-Type') eq 'application/json') {
-        my $json = $ua->content;
+    if ($res->header('Content-Type') and $res->header('Content-Type') =~ 'application/json') {
+        my $json = $res->decoded_content;
         $data = eval { $self->json->jsonToObj($json) };
         unless ($data) {
             # We tolerate bad JSON for errors,
@@ -156,9 +166,15 @@ sub query {
     }
 
     if ( $self->RaiseError ) {
-        croak $data->{message} if not $ua->success and ref $data eq 'HASH' and exists $data->{message}; # for 'Client Errors'
+        croak $data->{message} if not $res->is_success and ref $data eq 'HASH' and exists $data->{message}; # for 'Client Errors'
     }
-    
+
+    $self->_clear_pagination;
+    if ($res->header('link')) {
+        my @rel_strs = split ',', $res->header('link');
+        $self->_extract_link_url(\@rel_strs);
+    }
+
     ## be smarter
     if (wantarray) {
         return @$data if ref $data eq 'ARRAY';
@@ -168,11 +184,44 @@ sub query {
     return $data;
 }
 
+sub next_page {
+    my $self = shift;
+    return $self->query($self->next_url);
+}
+
+sub _clear_pagination {
+    my $self = shift;
+    foreach my $page (qw/first last prev next/) {
+        my $clearer = 'clear_' . $page . '_url';
+        $self->$clearer;
+    }
+    return 1;
+}
+
+sub _extract_link_url {
+    my ($self, $raw_strs) = @_;
+    foreach my $str (@$raw_strs) {
+        my ($link_url, $rel) = split ';', $str;
+
+        $link_url =~ s/^\s*//;
+        $link_url =~ s/^<//;
+        $link_url =~ s/>$//;
+
+        $rel =~ m/rel="(next|last|first|prev)"/;
+        $rel = $1;
+
+        my $url_attr = $rel . "_url";
+        $self->$url_attr($link_url);
+    }
+
+    return 1;
+}
+
 ## build methods on fly
 sub __build_methods {
     my $package = shift;
     my %methods = @_;
-    
+
     foreach my $m (keys %methods) {
         my $v = $methods{$m};
         my $url = $v->{url};
@@ -180,13 +229,15 @@ sub __build_methods {
         my $args = $v->{args} || 0; # args for ->query
         my $check_status = $v->{check_status};
         my $is_u_repo = $v->{is_u_repo}; # need auto shift u/repo
-        
-        $package->meta->add_method( $m => sub {
+
+        no strict 'refs';
+        no warnings 'once';
+        *{"${package}::${m}"} = sub {
             my $self = shift;
-            
+
             # count how much %s inside u
             my $n = 0; while ($url =~ /\%s/g) { $n++ }
-            
+
             ## if is_u_repo, both ($user, $repo, @args) or (@args) should be supported
             if ( ($is_u_repo or index($url, '/repos/%s/%s') > -1) and @_ < $n + $args) {
                 unshift @_, ($self->u, $self->repo);
@@ -195,7 +246,7 @@ sub __build_methods {
             # make url, replace %s with real args
             my @uargs = splice(@_, 0, $n);
             my $u = sprintf($url, @uargs);
-            
+
             # args for json data POST
             my @qargs = $args ? splice(@_, 0, $args) : ();
             if ($check_status) { # need check Response Status
@@ -207,11 +258,11 @@ sub __build_methods {
             } else {
                 return $self->query($method, $u, @qargs);
             }
-        } );
+        };
     }
 }
 
-no Any::Moose;
+no Moo::Role;
 
 1;
 __END__
@@ -224,7 +275,7 @@ Net::GitHub::V3::Query - Base Query role for Net::GitHub::V3
 
     package Net::GitHub::V3::XXX;
 
-    use Any::Moose;
+    use Moo;
     with 'Net::GitHub::V3::Query';
 
 =head1 DESCRIPTION
@@ -263,8 +314,18 @@ L<http://developer.github.com/>
 
 Refer L<Net::GitHub::V3>
 
+=item next_page
+
+Calls C<query> with C<next_url>. See L<Net::GitHub::V3>
+
 =back
 
+=head3 NG_DEBUG
+
+export NG_DEBUG=1 to view the request URL
+
+NG_DEBUG > 1 to view request/response string
+
 =head1 AUTHOR & COPYRIGHT & LICENSE
 
-Refer L<Net::GitHub>
\ No newline at end of file
+Refer L<Net::GitHub>
